package com.yfd.monitor.vmanager.gdw2019.play;

import cn.hutool.core.util.ObjUtil;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.yfd.monitor.common.StreamInfo;
import com.yfd.monitor.conf.UserSetting;
import com.yfd.monitor.conf.exception.ControllerException;
import com.yfd.monitor.conf.exception.SsrcTransactionNotFoundException;
import com.yfd.monitor.conf.security.JwtUtils;
import com.yfd.monitor.gdw2019.bean.Device;
import com.yfd.monitor.gdw2019.bean.SsrcTransaction;
import com.yfd.monitor.gdw2019.session.VideoStreamSessionManager;
import com.yfd.monitor.gdw2019.transmit.callback.DeferredResultHolder;
import com.yfd.monitor.gdw2019.transmit.callback.RequestMessage;
import com.yfd.monitor.gdw2019.transmit.cmd.impl.SIPCommander;
import com.yfd.monitor.media.zlm.ZLMRESTfulUtils;
import com.yfd.monitor.media.zlm.dto.MediaServerItem;
import com.yfd.monitor.service.IMediaServerService;
import com.yfd.monitor.service.IMediaService;
import com.yfd.monitor.service.IPlayService;
import com.yfd.monitor.storager.IRedisCatchStorage;
import com.yfd.monitor.storager.IVideoManagerStorage;
import com.yfd.monitor.vmanager.bean.*;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.async.DeferredResult;

import javax.servlet.http.HttpServletRequest;
import javax.sip.InvalidArgumentException;
import javax.sip.SipException;
import java.text.ParseException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

@Tag(name  = "国标设备点播")

@RestController
@RequestMapping("/api/play")
public class PlayController {

	private final static Logger logger = LoggerFactory.getLogger(PlayController.class);

	@Autowired
	private SIPCommander cmder;

	@Autowired
	private VideoStreamSessionManager streamSession;

	@Autowired
	private IVideoManagerStorage storager;

	@Autowired
	private IRedisCatchStorage redisCatchStorage;

	@Autowired
	private ZLMRESTfulUtils zlmresTfulUtils;

	@Autowired
	private DeferredResultHolder resultHolder;

	@Autowired
	private IPlayService playService;

	@Autowired
	private IMediaService mediaService;

	@Autowired
	private IMediaServerService mediaServerService;

	@Autowired
	private UserSetting userSetting;

	@Operation(summary = "开始点播")
	@Parameter(name = "deviceId", description = "设备国标编号", required = true)
	@Parameter(name = "channelId", description = "通道国标编号", required = true)
	@GetMapping("/start/{deviceId}/{channelId}")
	public DeferredResult<WVPResult<StreamContent>> play(HttpServletRequest request, @PathVariable String deviceId,
														 @PathVariable String channelId) {

		// 获取可用的zlm
		Device device = storager.queryVideoDevice(deviceId);
		MediaServerItem newMediaServerItem = playService.getNewMediaServerItem(device);
		boolean callsubSteam=false;
		String[] temp=channelId.split("_");
		DeferredResult<WVPResult<StreamContent>> result =
				new DeferredResult<>(userSetting.getPlayTimeout().longValue());
		if(temp.length==2){
			channelId=temp[0];
			if("sub".equals(temp[1])){
				callsubSteam = true;
				return playService.play_subStream(newMediaServerItem, deviceId, channelId);
			}
			System.out.println("进入了多屏方法");
		}
		// 获取可用的zlm
		if (device == null) {
			System.out.println("进入了机器人摄像机方法");
			List<Map> patrolDeviceList = storager.getRiisPatrolDeviceByCode(deviceId);
			if (patrolDeviceList.size() > 0) {
				Map map1 = patrolDeviceList.get(0);
				Map map = new HashMap();
				String type = map1.get("type").toString();
				if ("1".equals(type) || "2".equals(type) || "3".equals(type) || "4".equals(type)|| "13".equals(type)) {
					List<Map> riisPatrolDeviceByRobot = storager.getRiisPatrolDeviceByRobot(deviceId);
					map.putAll(riisPatrolDeviceByRobot.get(0));
				}else{
					map.putAll(map1);
				}
				String patroldevice_code = map.get("patroldevice_code").toString();
				// 通过摄像机本身的rtsp流播放视频
				String stream = String.format("%s_%s", patroldevice_code, deviceId);
				String app = "rtp";
				String url = map.get("rtspurl").toString();
				JSONObject list = zlmresTfulUtils.getMediaList(newMediaServerItem, app, stream);
				if (list == null || ObjUtil.isEmpty(list.getJSONArray("data"))) {
					JSONObject streamProxy = zlmresTfulUtils.addStreamProxy(newMediaServerItem, app, stream, url,
							false, true, "0");
					System.out.println(streamProxy.toJSONString());
				}
				WVPResult<StreamContent> result1 = new WVPResult<>();
				result1.setCode(0);
				result1.setMsg("成功！");
				String fmp4 = String.format("http://%s:%s/rtp/%s.live.mp4", newMediaServerItem.getIp(),
						newMediaServerItem.getHttpPort(),
						stream);
				String ws_flv = String.format("ws://%s:%s/rtp/%s.live.flv", newMediaServerItem.getIp(),
						newMediaServerItem.getHttpPort(),
						stream);

				result1.setData(new StreamContent(fmp4, ws_flv));
				result.setResult(result1);
				return result;
			}

		}

		RequestMessage msg = new RequestMessage();
		String key = DeferredResultHolder.CALLBACK_CMD_PLAY + deviceId + channelId;
		boolean exist = resultHolder.exist(key, null);
		msg.setKey(key);
		String uuid = UUID.randomUUID().toString();
		msg.setId(uuid);

		DeferredResultEx<WVPResult<StreamContent>> deferredResultEx = new DeferredResultEx<>(result);
		result.onTimeout(() -> {
			logger.info("点播接口等待超时");
			// 释放rtpserver
			WVPResult<StreamInfo> wvpResult = new WVPResult<>();
			wvpResult.setCode(ErrorCode.ERROR100.getCode());
			wvpResult.setMsg("点播超时");
			msg.setData(wvpResult);
			resultHolder.invokeResult(msg);
		});
		// TODO 在点播未成功的情况下在此调用接口点播会导致返回的流地址ip错误
		deferredResultEx.setFilter(result1 -> {
			WVPResult<StreamInfo> wvpResult1 = (WVPResult<StreamInfo>)result1;
			WVPResult<StreamContent> resultStream = new WVPResult<>();
			resultStream.setCode(wvpResult1.getCode());
			resultStream.setMsg(wvpResult1.getMsg());
			if (wvpResult1.getCode() == ErrorCode.SUCCESS.getCode()) {
				StreamInfo data = wvpResult1.getData().clone();
				if (userSetting.getUseSourceIpAsStreamIp()) {
					data.channgeStreamIp(request.getLocalName());
				}
				resultStream.setData(new StreamContent(wvpResult1.getData()));
			}
			return resultStream;
		});


		// 录像查询以channelId作为deviceId查询
		resultHolder.put(key, uuid, deferredResultEx);

		if (!exist) {
			playService.play(newMediaServerItem, deviceId, channelId, null, null, null);
		}
		return result;
	}

	@Operation(summary = "停止点播")
	@Parameter(name = "deviceId", description = "设备国标编号", required = true)
	@Parameter(name = "channelId", description = "通道国标编号", required = true)
	@GetMapping("/stop/{deviceId}/{channelId}")
	public JSONObject playStop(@PathVariable String deviceId, @PathVariable String channelId) {

		logger.debug(String.format("设备预览/回放停止API调用，streamId：%s_%s", deviceId, channelId));

		if (deviceId == null || channelId == null) {
			throw new ControllerException(ErrorCode.ERROR400);
		}

		Device device = storager.queryVideoDevice(deviceId);
		MediaServerItem newMediaServerItem = playService.getNewMediaServerItem(device);
		String[] temp = channelId.split("_");
		if (temp.length == 2) {
			if ("sub".equals(temp[1])) {
				String app = "rtp";
				String stream = String.format("%s_%s", deviceId, channelId);
				return zlmresTfulUtils.closeStreams(newMediaServerItem, app, stream);
			}
			System.out.println("进入了多屏方法");
		}
		if (device == null) {
			System.out.println("进入了机器人摄像机方法");
			List<Map> patrolDeviceList = storager.getRiisPatrolDeviceByCode(deviceId);
			if (patrolDeviceList.size() > 0) {
				Map map1 = patrolDeviceList.get(0);
				Map map = new HashMap();
				String type = map1.get("type").toString();
				if ("1".equals(type) || "2".equals(type) || "3".equals(type) || "4".equals(type) || "13".equals(type)) {
					List<Map> riisPatrolDeviceByRobot = storager.getRiisPatrolDeviceByRobot(deviceId);
					map.putAll(riisPatrolDeviceByRobot.get(0));
				} else {
					map.putAll(map1);
				}
				String patroldevice_code = map.get("patroldevice_code").toString();
				// 通过摄像机本身的rtsp流播放视频
				String stream = String.format("%s_%s", patroldevice_code, deviceId);
				String app = "rtp";
				JSONObject result = zlmresTfulUtils.closeStreams(newMediaServerItem, app, stream);
				if (ObjUtil.isNotEmpty(result.getIntValue("count_closed")) && (result.getIntValue("count_closed") > 0)) {
					return result;
				} else {
					return null;
				}
			}
			throw new ControllerException(ErrorCode.ERROR100.getCode(), "设备[" + deviceId + "]不存在");
		}
		if("Thermal Image".equals(device.getManufacturer())){//大力的红外摄像头（临时处理）
			String key=String.format("__defaultVhost__/rtp/%s_%s",deviceId,channelId);
			String stream=String.format("%s_%s",deviceId,channelId);
			String app="rtp";
			JSONObject result=zlmresTfulUtils.closeStreams(newMediaServerItem,app,stream);
			if(ObjUtil.isNotEmpty(result.getIntValue("count_closed"))&&(result.getIntValue("count_closed")>0)){
				return result;
			}else{
				return null;
			}
		}

		StreamInfo streamInfo = redisCatchStorage.queryPlayByDevice(deviceId, channelId);
		if (streamInfo == null) {
			throw new ControllerException(ErrorCode.ERROR100.getCode(), "点播未找到");
		}

		try {
			logger.warn("[停止点播] {}/{}", device.getDeviceId(), channelId);
			cmder.streamByeCmd(device, channelId, streamInfo.getStream(), null, null);
		} catch (InvalidArgumentException | SipException | ParseException | SsrcTransactionNotFoundException e) {
			logger.error("[命令发送失败] 停止点播， 发送BYE: {}", e.getMessage());
			throw new ControllerException(ErrorCode.ERROR100.getCode(), "命令发送失败: " + e.getMessage());
		}
		redisCatchStorage.stopPlay(streamInfo);

		storager.stopPlay(streamInfo.getDeviceID(), streamInfo.getChannelId());
		JSONObject json = new JSONObject();
		json.put("deviceId", deviceId);
		json.put("channelId", channelId);
		return json;
	}

	/**
	 * 将不是h264的视频通过ffmpeg 转码为h264 + aac
	 * @param streamId 流ID
	 */
	@Operation(summary = "将不是h264的视频通过ffmpeg 转码为h264 + aac")
	@Parameter(name = "streamId", description = "视频流ID", required = true)
	@PostMapping("/convert/{streamId}")
	public JSONObject playConvert(@PathVariable String streamId) {
		StreamInfo streamInfo = redisCatchStorage.queryPlayByStreamId(streamId);
		if (streamInfo == null) {
			streamInfo = redisCatchStorage.queryPlayback(null, null, streamId, null);
		}
		if (streamInfo == null) {
			logger.warn("视频转码API调用失败！, 视频流已经停止!");
			throw new ControllerException(ErrorCode.ERROR100.getCode(), "未找到视频流信息, 视频流可能已经停止");
		}
		MediaServerItem mediaInfo = mediaServerService.getOne(streamInfo.getMediaServerId());
		JSONObject rtpInfo = zlmresTfulUtils.getRtpInfo(mediaInfo, streamId);
		if (!rtpInfo.getBoolean("exist")) {
			logger.warn("视频转码API调用失败！, 视频流已停止推流!");
			throw new ControllerException(ErrorCode.ERROR100.getCode(), "未找到视频流信息, 视频流可能已停止推流");
		} else {
			String dstUrl = String.format("rtmp://%s:%s/convert/%s", "127.0.0.1", mediaInfo.getRtmpPort(),
					streamId );
			String srcUrl = String.format("rtsp://%s:%s/rtp/%s", "127.0.0.1", mediaInfo.getRtspPort(), streamId);
			JSONObject jsonObject = zlmresTfulUtils.addFFmpegSource(mediaInfo, srcUrl, dstUrl, "1000000", true, false, null);
			logger.info(jsonObject.toJSONString());
			if (jsonObject != null && jsonObject.getInteger("code") == 0) {
				JSONObject data = jsonObject.getJSONObject("data");
				if (data != null) {
					JSONObject result = new JSONObject();
					result.put("key", data.getString("key"));
					StreamInfo streamInfoResult = mediaService.getStreamInfoByAppAndStreamWithCheck("convert", streamId, mediaInfo.getId(), false);
					result.put("StreamInfo", streamInfoResult);
					return result;
				}else {
					throw new ControllerException(ErrorCode.ERROR100.getCode(), "转码失败");
				}
			}else {
				throw new ControllerException(ErrorCode.ERROR100.getCode(), "转码失败");
			}
		}
	}

	/**
	 * 结束转码
	 */
	@Operation(summary = "结束转码")
	@Parameter(name = "key", description = "视频流key", required = true)
	@Parameter(name = "mediaServerId", description = "流媒体服务ID", required = true)
	@PostMapping("/convertStop/{key}")
	public void playConvertStop(@PathVariable String key, String mediaServerId) {
		if (mediaServerId == null) {
			throw new ControllerException(ErrorCode.ERROR400.getCode(), "流媒体：" + mediaServerId + "不存在" );
		}
		MediaServerItem mediaInfo = mediaServerService.getOne(mediaServerId);
		if (mediaInfo == null) {
			throw new ControllerException(ErrorCode.ERROR100.getCode(), "使用的流媒体已经停止运行" );
		}else {
			JSONObject jsonObject = zlmresTfulUtils.delFFmpegSource(mediaInfo, key);
			logger.info(jsonObject.toJSONString());
			if (jsonObject != null && jsonObject.getInteger("code") == 0) {
				JSONObject data = jsonObject.getJSONObject("data");
				if (data == null || data.getBoolean("flag") == null || !data.getBoolean("flag")) {
					throw new ControllerException(ErrorCode.ERROR100 );
				}
			}else {
				throw new ControllerException(ErrorCode.ERROR100 );
			}
		}
	}

//	@Operation(summary = "语音广播命令")
//	@Parameter(name = "deviceId", description = "设备国标编号", required = true)
//    @GetMapping("/broadcast/{deviceId}")
//    @PostMapping("/broadcast/{deviceId}")
//    public DeferredResult<String> broadcastApi(@PathVariable String deviceId) {
//        if (logger.isDebugEnabled()) {
//            logger.debug("语音广播API调用");
//        }
//        Device device = storager.queryVideoDevice(deviceId);
//		DeferredResult<String> result = new DeferredResult<>(3 * 1000L);
//		String key  = DeferredResultHolder.CALLBACK_CMD_BROADCAST + deviceId;
//		if (resultHolder.exist(key, null)) {
//			result.setResult("设备使用中");
//			return result;
//		}
//		String uuid  = UUID.randomUUID().toString();
//        if (device == null) {
//
//			resultHolder.put(key, key,  result);
//			RequestMessage msg = new RequestMessage();
//			msg.setKey(key);
//			msg.setId(uuid);
//			JSONObject json = new JSONObject();
//			json.put("DeviceID", deviceId);
//			json.put("CmdType", "Broadcast");
//			json.put("Result", "Failed");
//			json.put("Description", "Device 不存在");
//			msg.setData(json);
//			resultHolder.invokeResult(msg);
//			return result;
//		}
//		try {
//			cmder.audioBroadcastCmd(device, (event) -> {
//				RequestMessage msg = new RequestMessage();
//				msg.setKey(key);
//				msg.setId(uuid);
//				JSONObject json = new JSONObject();
//				json.put("DeviceID", deviceId);
//				json.put("CmdType", "Broadcast");
//				json.put("Result", "Failed");
//				json.put("Description", String.format("语音广播操作失败，错误码： %s, %s", event.statusCode, event.msg));
//				msg.setData(json);
//				resultHolder.invokeResult(msg);
//			});
//		} catch (InvalidArgumentException | SipException | ParseException e) {
//			logger.error("[命令发送失败] 语音广播: {}", e.getMessage());
//			throw new ControllerException(ErrorCode.ERROR100.getCode(), "命令发送失败: " + e.getMessage());
//		}
//
//		result.onTimeout(() -> {
//			logger.warn("语音广播操作超时, 设备未返回应答指令");
//			RequestMessage msg = new RequestMessage();
//			msg.setKey(key);
//			msg.setId(uuid);
//			JSONObject json = new JSONObject();
//			json.put("DeviceID", deviceId);
//			json.put("CmdType", "Broadcast");
//			json.put("Result", "Failed");
//			json.put("Error", "Timeout. Device did not response to broadcast command.");
//			msg.setData(json);
//			resultHolder.invokeResult(msg);
//		});
//		resultHolder.put(key, uuid, result);
//		return result;
//	}


	@Operation(summary = "语音广播命令")
	@Parameter(name = "deviceId", description = "设备国标编号", required = true)
	@Parameter(name = "deviceId", description = "通道国标编号", required = true)
	@Parameter(name = "timeout", description = "推流超时时间(秒)", required = true)
	@GetMapping("/broadcast/{deviceId}/{channelId}")
	@PostMapping("/broadcast/{deviceId}/{channelId}")
	public AudioBroadcastResult broadcastApi(@PathVariable String deviceId, @PathVariable String channelId, Integer timeout, Boolean broadcastMode) {
		if (logger.isDebugEnabled()) {
			logger.debug("语音广播API调用");
		}
		Device device = storager.queryVideoDevice(deviceId);
		if (device == null) {
			throw new ControllerException(ErrorCode.ERROR400.getCode(), "未找到设备： " + deviceId);
		}
		if (channelId == null) {
			throw new ControllerException(ErrorCode.ERROR400.getCode(), "未找到通道： " + channelId);
		}

		return playService.audioBroadcast(device, channelId, broadcastMode);

	}


	@Operation(summary = "停止语音广播")
	@Parameter(name = "deviceId", description = "设备Id", required = true)
	@Parameter(name = "channelId", description = "通道Id", required = true)
	@GetMapping("/broadcast/stop/{deviceId}/{channelId}")
	@PostMapping("/broadcast/stop/{deviceId}/{channelId}")
	public void stopBroadcast(@PathVariable String deviceId, @PathVariable String channelId) {
		if (logger.isDebugEnabled()) {
			logger.debug("停止语音广播API调用");
		}
		//		try {
		//			playService.stopAudioBroadcast(deviceId, channelId);
		//		} catch (InvalidArgumentException | ParseException  | SipException e) {
		//			logger.error("[命令发送失败] 停止语音: {}", e.getMessage());
		//			throw new ControllerException(ErrorCode.ERROR100.getCode(), "命令发送失败: " +  e.getMessage());
		//		}
		playService.stopAudioBroadcast(deviceId, channelId);
	}

	@Operation(summary = "获取所有的ssrc")
	@GetMapping("/ssrc")
	public JSONObject getSSRC() {
		if (logger.isDebugEnabled()) {
			logger.debug("获取所有的ssrc");
		}
		JSONArray objects = new JSONArray();
		List<SsrcTransaction> allSsrc = streamSession.getAllSsrc();
		for (SsrcTransaction transaction : allSsrc) {
			JSONObject jsonObject = new JSONObject();
			jsonObject.put("deviceId", transaction.getDeviceId());
			jsonObject.put("channelId", transaction.getChannelId());
			jsonObject.put("ssrc", transaction.getSsrc());
			jsonObject.put("streamId", transaction.getStream());
			objects.add(jsonObject);
		}

		JSONObject jsonObject = new JSONObject();
		jsonObject.put("data", objects);
		jsonObject.put("count", objects.size());
		return jsonObject;
	}

}

